
#import <Cocoa/Cocoa.h>
#import "PreferenceController.h"

NSString *BWShowAlertKey = @"Show Alert";
NSString *BWBounceIconKey = @"Bounce Icon";
NSString *BWMessageKey = @"Message";
NSString *BWAlertTimesKey = @"Alert Times";

@implementation PreferenceController

- (NSMutableDictionary *) makeFactoryDefaults
{
    NSNumber *trueBool = [NSNumber numberWithBool: YES];
    NSNumber *falseBool = [NSNumber numberWithBool: NO];
    NSMutableDictionary *defaultValues = [NSMutableDictionary dictionary];
    
    [defaultValues setObject: trueBool forKey: BWShowAlertKey];
    [defaultValues setObject: falseBool forKey: BWBounceIconKey];
    [defaultValues setObject: @"go for a walk around the block you spaz" 
                        forKey: BWMessageKey];
    {
        NSData *data;
        alertTimes = [[NSMutableSet setWithCapacity: 12] retain]; // 5 minute increments
        [alertTimes addObject: [NSNumber numberWithInt: 50]]; // default to 0:50
        data = [NSArchiver archivedDataWithRootObject: alertTimes];
        [defaultValues setObject: data forKey: BWAlertTimesKey];
    }
   
    return (defaultValues);
    
} // makeFactoryDefaults


- (id) init
{
    if (self = [super initWithWindowNibName: @"Preferences"]) {
        NSMutableDictionary *defaultValues = [self makeFactoryDefaults];
        [[NSUserDefaults standardUserDefaults] registerDefaults: defaultValues];
        [self setWindowFrameAutosaveName: @"PreferenceWindow"];
    }
    
    return (self);
    
} // init



- (void) updateUI
{
    // populate window with initial values
    BOOL onOff;
    NSString *messagePref;
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        
    onOff = [[defaults objectForKey: BWShowAlertKey] boolValue];
    
    [alertButton setState: onOff];
    [message setEnabled: onOff];
    
    onOff = [[defaults objectForKey: BWBounceIconKey] boolValue];
    [bounceButton setState: onOff];
    
    messagePref = [defaults objectForKey: BWMessageKey];
    [message setStringValue: messagePref];
    
    // walk the box turning things on
    {
        NSData *alertTimesAsData = [defaults objectForKey: BWAlertTimesKey];
        int tags[] = { 0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55 };
        int i;
        
        if (alertTimesAsData != nil) {
            // ??? might be leaking
            NSLog (@"have archived alertTimes");
            alertTimes = [[NSUnarchiver unarchiveObjectWithData: alertTimesAsData] retain];
        }
        
        for (i = 0; i < sizeof(tags) / sizeof(int); i++) {
            int tag = tags[i];
            NSButton *button = [hoursBox viewWithTag: tag];
                        
            if ([alertTimes containsObject: [NSNumber numberWithInt: tag]]) {
                [button setState: YES];
            } else {
                [button setState: NO];
            }
        }
     }

} // updateUI


- (void) windowDidLoad
{
    NSLog (@"windowDidLoad");
    [self updateUI];    
} // windowDidLoad



- (IBAction) changeAlertTime: (id) sender
{
    int tag = [sender tag];
    BOOL onOff = [sender state];
    
    if (onOff) {
        [alertTimes addObject: [NSNumber numberWithInt: tag]];
    } else {
        [alertTimes removeObject: [NSNumber numberWithInt: tag]];
    }
    
    {
        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        NSData *data = [NSArchiver archivedDataWithRootObject: alertTimes];
        [defaults setObject: data forKey: BWAlertTimesKey];
    }
        
} // changeAlertTime


- (IBAction) changeShowAlert: (id) sender
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    
    if ([sender state]) {
        [message setEnabled: YES];
    } else {
        [message setEnabled: NO];
    }
    
    [defaults setObject: [NSNumber numberWithBool: [sender state]] forKey: BWShowAlertKey];
    
} // changeShowAlert


- (IBAction) changeIconBounce: (id) sender
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    
    NSLog (@"changeIconBounce");

    [defaults setObject: [NSNumber numberWithBool: [sender state]] forKey: BWBounceIconKey];

} // changeIconBounce


- (IBAction) changeAlertMessage: (id) sender
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSLog (@"changeAlertMessage");
    
    [defaults setObject: [sender stringValue] forKey: BWMessageKey];
} // changeAlertMessage


- (IBAction) factoryDefaults: (id) sender
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    
    [defaults removeObjectForKey: BWShowAlertKey];
    [defaults removeObjectForKey: BWBounceIconKey];
    [defaults removeObjectForKey: BWMessageKey];
    [defaults removeObjectForKey: BWAlertTimesKey];
    
    {
        // !!! grody!  this sets the alertTimes instancevar to thefactory default
        NSMutableDictionary *dict = [self makeFactoryDefaults];
        
    }
    
    [self updateUI];
    NSLog (@"factoryDefaults: %@", alertTimes);
} // factoryDefaults


- (BOOL) shouldBounce
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    BOOL onOff = [[defaults objectForKey: BWBounceIconKey] boolValue];
    return (onOff);
} // shouldBounce


- (BOOL) shouldAlert
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    BOOL onOff = [[defaults objectForKey: BWShowAlertKey] boolValue];
    return (onOff);

} // shouldAlert


- (NSString *) alertMessage
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSString *string = [defaults objectForKey: BWMessageKey];
    return (string);

} // alertMessage


- (NSSet *) alertTimes
{
    return (alertTimes);
} // alertTimes


- (void) dealloc
{
    [[NSUserDefaults standardUserDefaults] synchronize];
    [super dealloc];
} // dealloc


@end // PreferenceController
